
/* 
Word Clock by Elliot
www.elliotmade.com


You may need to select atmega 328p (old bootloader) in order to program

Clock operation:
	button 1 - mode/set. short click incriments the animation in clock mode
		long press will enter or save and exit from clock set mode
	button 2 - up.  short click will incriment the color in clock mode,
		or digit in set clock mode.  Long press enters and exits demo mode (single click exits also)
	button 3 - down/cancel.  short click will decrement the color in clock mode, long press will go 
		back to clock mode and not save anything

eeprom address usage:
	2: color index int
	5: animation mode
 */

/* How the display part works
  
  an array with 2 values for every LED is created
  the color and on/off value is stored and updated here by copying the on/off and color
  from this array to the fastled array
  for regular updates the display is updated all at once
  this is typically used for the time or other stating things to display
  
  typical way to update the display: 
	clear the grid array
	first set the grid array [0] values to 1 or 0 for on/off
	second set the grid array [1] values to the colorindex for the desired color - updatecolor function
	third update the dimmer value
 
 
   Color byte in this array is an index for the colors aray
   
  Animations should typically be triggered on a time change - from one five minute display to the next
  Animations may display directly and not update the grid array
 
 */

//libraries
#include <Wire.h>        	//http://arduino.cc/en/Reference/Wire (included with Arduino IDE)
#include <FastLED.h>  		//fastled.io
#include <EEPROM.h>
#include <RTClib.h>
#include <OneButton.h>
#include <SimpleTimer.h>


/* notes for my arduino wiring:
 * pin 2: top button (set)
 * pin 3: middle button (up)
 * pin 4: bottom button (down)
 * pin 6: LED strand data
 * pin A0: photosensor - voltage divider with 10k resistor
 * pin A4: RTC Module SDA
 * pin A5: RTC Module SCL
 */

 //pins
  int button1pin = 2;
  int button2pin = 3;
  int button3pin = 4;

 //FastLED
 #define NUM_LEDS 121
 #define NUM_COLORS 9 //standard colors for user to switch between
 #define MAX_COLOR 10 //allow variable to go past the array index.  9=rainbow 10=gradient
 #define DATA_PIN 6
 CRGB leds[NUM_LEDS];
 CRGB colors[NUM_COLORS]; //only used for copying the color value to the main leds array
 CRGB refpixel[1]; //use during animation to store a temporary value for comparison


//buttons
  OneButton button1(button1pin, true);
  OneButton button2(button2pin, true);
  OneButton button3(button3pin, true);

//timer
SimpleTimer timer;

//dimmer
  int photoResistor=A0;
  int photoRead;
  int dimmer=255;
  const byte readcount = 5;
  byte dimmervalues[readcount];
  byte dimmerindex = 0;
  int dimmertotal = 0;
  float dimmermult = 1.2;
  

 //led matrix variable
 byte grid[121][2]; //first is the number of leds, second consists of [on/off, color index}

//DS3231 RTC
  RTC_DS3231 rtc;

//Operation modes
int mode = 0; //starting mode for the clock
#define CLOCK_MODE 0
#define SET_TIME_MODE 1
#define DEMO_MODE 2
#define MAX_MODE 3



//modes inside of time set
int setMode = 0;  //starting mode for setting the time
#define HH 0
#define MM 1
#define SETMAX 2

//animation mode for displaying the time
int animmode;

//time variables - for current time
  int mytimemonth;
  int mytimeday;
  int mytimehr;
  int mytimemin;
  int mytimesec;
  int mydayofweek;

//time variables - for setting new time
  int newtimemonth;
  int newtimeday;
  int newtimehr;
  int newtimemin;
  int newtimesec;
  int newtimeyear;

//time variables - for deciding when to update display
  int prevtimemin;
  bool timechanged;

 //other variables
  int colorindex;
  int setanimation;
  #define NUM_ANIMATIONS 4
  
//digits.  First number is the length of the array
byte digitzero[] = {17, 23, 24, 25, 43, 39, 44, 48, 65, 61, 66, 70, 87, 83, 89, 90, 91};
byte digitone[] = {11, 24, 42, 41, 46, 63, 68, 85, 89, 90, 91};
byte digittwo[] = {15, 23, 24, 25, 43, 39, 48, 62, 68, 86, 88, 89, 90, 91, 92};
byte digitthree[] = {15, 23, 24, 25, 43, 39, 48, 63, 62, 70, 87, 83, 89, 90, 91};
byte digitfour[] = {15, 25, 41, 40, 45, 47, 65, 62, 66, 67, 68, 69, 70, 84, 91};
byte digitfive[] = {18, 22, 23, 24, 25, 26, 43, 44, 45, 46, 47, 61, 70, 87, 83, 89, 90, 91};
byte digitsix[] = {16, 24, 25, 42, 44, 65, 64, 63, 62, 66, 70, 87, 83, 89, 90, 91};
byte digitseven[] = {12, 22, 23, 24, 25, 26, 39, 47, 63, 68, 85, 90};
byte digiteight[] = {18, 23, 24, 25, 43, 39, 44, 48, 64, 63, 62, 66, 70, 87, 83, 89, 90, 91};
byte digitnine[] = {16, 23, 24, 25, 43, 39, 44, 48, 64, 63, 62, 61, 70, 84, 89, 90};


//cycle through colors and animations in demo mode (pretty sloppy at this time)?
boolean superdemo = false;
byte democounter = 0;

 
void setup() {
//Initialize serial communication
Serial.begin(9600); 

FastLED.addLeds<NEOPIXEL, DATA_PIN>(leds, NUM_LEDS); //initialise fastled

  //list of standard colors
  colors[0] = CRGB::Red;
  colors[1] = CRGB::OrangeRed;
  colors[2] = CRGB::Yellow;
  colors[3] = CRGB::Green;
  colors[4] = CRGB::Blue;
  colors[5] = CRGB::Teal;
  colors[6] = CRGB::Purple;
  colors[7] = CRGB::Chocolate;
  colors[8] = CRGB::White;
  
  refpixel[0] = CRGB::White;


//read saved settings from eeprom
 EEPROM.get(2, colorindex);
 EEPROM.get(5, setanimation);
 Serial.print("Stored Color: ");
 Serial.print(colorindex);
 Serial.print(" Stored Animation: ");
 Serial.println(setanimation);


pinMode(photoResistor, INPUT);

//start rtc
 if (! rtc.begin()) {
    Serial.println("Couldn't find RTC"); //program LEDs to show "NO CLOCK"
    while (1);
  }

  //set up button functions
  button1.attachClick(click1);
  button1.attachLongPressStart(longPressStart1);

  button2.attachClick(click2);
  button2.attachLongPressStart(longPressStart2);

  button3.attachClick(click3);
  button3.attachLongPressStart(longPressStart3);

	//timer
	timer.setInterval(100, applydimmer);

	//initialize dimmer readings at full brightness
	for (byte a = 0; a < readcount; a++) {
			dimmervalues[a] = 255;
	}
  
  
  applycolorstogrid(colorindex); //set the initial color
  boop(); //say hello
  Serial.println("Boop");
  gettime(); //get the initial time
  refreshtimetogrid(); //put it in the grid
  runanimation(); //and display it
  




} //end setup

void loop() {

	timer.run();
	button1.tick();
	button2.tick();
	button3.tick();

	if (mode == CLOCK_MODE){ //only do stuff if it's in clock mode
		
		checktimestate();
		if (timechanged == true){
			runanimation();
		}
	}
	if (mode == DEMO_MODE) { //accelerate time so the display changes more rapidly
		Serial.print(mytimehr);
		Serial.print(":");
		Serial.println(mytimemin);
		
		checktimestatedemo();
		if (timechanged == true){
			runanimation();
		}
		delay(300);
	}
	
}

void runanimation(){ //updating the time is done in each animation, not here anymore
  //cleargrid();
  //refreshtimetogrid();
  applydimmer;
  switch (setanimation) {
	  case 1:
		animationnone();
	  break;
	  case 2:
		animationfade();
	  break;
	  case 3:
		animationfadeall();
	  break;
	  case 4:
		animationzigzag();
	  break;
	  // case 5:
		// animationzigzagrainbow();
	  // break;
  }
  
  Serial.println("Animation just ran. ");
}

void click1() {
  Serial.println("Button 1 click.");
    // cycle the modes upon click.  Do it within the clock set mode if we're in set mode
	
	if (mode == CLOCK_MODE or mode == DEMO_MODE) {
		setanimation++;
		if (setanimation >= NUM_ANIMATIONS + 1) {
			setanimation = 1;
		}
		EEPROM.put(5, setanimation);
		Serial.print("Animation: ");
		Serial.print(setanimation);
		Serial.print(". ");
		boop(); //visual feedback
		refreshtimetogrid();
		runanimation();
	}

	if (mode == SET_TIME_MODE){
	setMode++;
		if (setMode >= SETMAX) {
			setMode = HH;
		}
	updatesettimedisplay();
	}

      
  Serial.print("Mode: ");
  Serial.print(mode);
  Serial.print(", Set Mode: ");
  Serial.println(setMode);
}

void longPressStart1() {
  Serial.println("Button 1 long press.");
  //if not in set mode, change to set mode and set variables to start
  //if in set mode, set the clock using variables and change back to hours mode
      if (mode == SET_TIME_MODE) {
        //set the time
        saveNewTime();
		boop(); //visual feedback
        //then change back to word mode
        mode = CLOCK_MODE;
		gettime();
		runanimation();
      }
      else {
        //change to time set mode, reset the time set mode count, get the current values for the variables
        DateTime now = rtc.now();
        newtimemonth=now.month();
        newtimeday=now.day();
        newtimehr=now.hour();
        newtimemin=now.minute();
        newtimesec=now.second();
        newtimeyear=now.year();
        setMode = HH;
        mode = SET_TIME_MODE;
		updatesettimedisplay();
      }
        
}

void click2() {
  Serial.println("Button 2 click.");
    // increase the relevant counter depending on the mode

      if (mode == SET_TIME_MODE){
        //month
        // if (setMode == MO) {
            // newtimemonth++;
            // if (newtimemonth >= 13)
                // newtimemonth = 1;
                // Serial.print("Month: ");
                // Serial.println(newtimemonth);
        // }
        // day
        // if (setMode == DD) {
            // newtimeday++;
            // if (newtimeday >= 32)
                // newtimeday = 1;
                // Serial.print("Day: ");
                // Serial.println(newtimeday);
        // }
        // year
        // if (setMode == YY) {
            // newtimeyear++;
            // if (newtimeyear >= 2100)
                // newtimeyear = 2100;
                // Serial.print("Year: ");
                // Serial.println(newtimeyear);
        // }
        //hour
        if (setMode == HH) {
            newtimehr++;
            if (newtimehr >= 24)
                newtimehr = 0;
                Serial.print("Hour: ");
                Serial.println(newtimehr);
        }
        //minute
        if (setMode == MM) {
            newtimemin++;
            if (newtimemin >= 60)
                newtimemin = 0;
                Serial.print("Minute: ");
                Serial.println(newtimemin);
        }
		updatesettimedisplay();
      } //this is the end of what to do if it's in set mode
	  
	  else if (mode == DEMO_MODE) {
		mode = CLOCK_MODE;
		}	

		// add something here to cycle colors if not in set the clock mode
      else {
        colorindex++;
      
	  if (colorindex >= MAX_COLOR)
          colorindex = 0; 
      Serial.print("Colorindex: ");
      Serial.println(colorindex);
      EEPROM.put(2, colorindex);
	  applycolorstogrid(colorindex);
	  updatedisplaynormal();
      }
      

}

void click3() {
  Serial.println("Button 3 click.");
    // increase the relevant counter depending on the mode

      if (mode == SET_TIME_MODE){
        //month
        // if (setMode == MO) {
            // newtimemonth--;
            // if (newtimemonth <= 0)
                // newtimemonth = 12;
                // Serial.print("Month: ");
                // Serial.println(newtimemonth);
        // }
        // day
        // if (setMode == DD) {
            // newtimeday--;
            // if (newtimeday <= 0)
                // newtimeday = 31;
                // Serial.print("Day: ");
                // Serial.println(newtimeday);
        // }
        // year
        // if (setMode == YY) {
            // newtimeyear--;
            // if (newtimeyear <= 2018)
                // newtimeyear = 2018;
                // Serial.print("Year: ");
                // Serial.println(newtimeyear);
        // }
        //hour
        if (setMode == HH) {
            newtimehr--;
            if (newtimehr <= -1)
                newtimehr = 23;
                Serial.print("Hour: ");
                Serial.println(newtimehr);
        }
        //minute
        if (setMode == MM) {
            newtimemin--;
            if (newtimemin <= -1)
                newtimemin = 59;
                Serial.print("Minute: ");
                Serial.println(newtimemin);
        }
		updatesettimedisplay();
      } //this is the end of what to do if it's in set mode

// add something here to cycle colors if not in set the clock mode
      else {
        colorindex--;
      if (colorindex <= -1)
          colorindex = MAX_COLOR - 1; 
      Serial.print("Colorindex: ");
      Serial.println(colorindex);
      EEPROM.put(2, colorindex);
	  applycolorstogrid(colorindex);
	  updatedisplaynormal();
      }
      

}

void longPressStart2() {
	Serial.print("Button 2 long press.  Mode:");
	boop(); //visual feedback
	gettime();
	refreshtimetogrid();
	runanimation();
	
	if (mode == CLOCK_MODE) {
		mode = DEMO_MODE;
	}
	else if (mode == DEMO_MODE) {
		mode = CLOCK_MODE;
	}
	Serial.println(mode);
}

void longPressStart3() {
  //long press to go back to word mode, to cancel setting time if you change your mind
  Serial.println("Button 3 long press.");
  boop(); //visual feedback
  mode = CLOCK_MODE;
  runanimation();
}

void updatedimmervalue(){
	//read the photoresistor, average the last five values
	
	photoRead = map(constrain(analogRead(photoResistor) * dimmermult, 0, 1023), 0, 1023, 10, 255);  //get the value, multiplier to adjust
	
	dimmervalues[dimmerindex] = photoRead; //update the array
	
	dimmerindex++; 					//incriment the index for next time
	if (dimmerindex >= readcount) { //but wrap to zero if it's beyond the limit
		dimmerindex = 0;
	}
	
	dimmertotal = 0; 					//wipe the old average
	for (int m = 0; m <= readcount; m++) { 	//then sum the readings up
		dimmertotal = dimmertotal + dimmervalues[m];
	}
	
	dimmer = dimmertotal / readcount; 	//and calculate a new average
	
	 Serial.print("Dimmer: ");
	 Serial.println(dimmer);
	
	//dimmer = 255;  //temporary
	
}

void applydimmer() { //apply dimmer to all LEDS

	updatedimmervalue();

  for (int f = 0;f < NUM_LEDS;f++){
    leds[f].maximizeBrightness();
    leds[f] %= dimmer;
  }

FastLED.show();
}

void checktimestate() { //if this is true, the time display needs to be updated.
  //min 0-59
  prevtimemin = mytimemin;
  gettime();
 
  int mindiff = mytimemin - prevtimemin;
  mindiff = abs(mindiff);
  if (timechanged == true) {
	  timechanged = false;
  }
  else {
	  if (mindiff > 0 and mytimemin % 5 == 0 and mytimemin != prevtimemin) {
		timechanged = true;
		
		//Serial.print("The time has changed. ");
	  }
	  else {
		//Serial.print("The time has not changed.  ");
		timechanged = false;
	  }
  }
}

void checktimestatedemo() { //if this is true, the time display needs to be updated.  "time" will be advanced by one minute every time it's called
  //min 0-59
  
  prevtimemin = mytimemin;
  mytimemin++;
  
  if (mytimemin >= 60) {
	mytimemin = 0;
	mytimehr++;
  }
  
  if (mytimehr >= 24) {
	  mytimehr = 0;
  }
  
 
  int mindiff = mytimemin - prevtimemin;
  mindiff = abs(mindiff);
  if (timechanged == true) {
	  timechanged = false;
  }
  else {
	  if (mindiff > 0 and mytimemin % 5 == 0 and mytimemin != prevtimemin) {
		timechanged = true;
		
		//Serial.print("The time has changed. ");
		
		//if we want to cycle colors and animations
		if (superdemo == true) {
			democounter++;
			click3(); //change the color every time	
			if (democounter >= 3) { //change animation every x time changes
				democounter = 0;
				click1(); //change the animation			
			}
		}	
	  }
	  else {
		//Serial.print("The time has not changed.  ");
		timechanged = false;
	  }
  }
}

void saveNewTime() {
  Serial.println("Attempting to save time to RTC");
  DateTime now = rtc.now();
  rtc.adjust(DateTime(newtimeyear, newtimemonth, newtimeday, newtimehr, newtimemin, now.second()));
  
}

void animationnone() {
  cleargrid();
  refreshtimetogrid();
  applycolorstogrid(colorindex);
  updatedisplaynormal();
}

void animationfade() {
	int tempdimmer = dimmer; //save the previous dimmer setting
	int dimincriment;
	
	
	
	dimincriment = dimmer / 21;
	//fade out
	cleargrid();
	refreshtimetogrid();
	
	//fade out the LEDS that are no longer on in the grid array
	for (int j = 0; j < 21; j++) { 
		dimmer = dimmer - dimincriment;
		for (int k = 0; k < NUM_LEDS; k++) {
			if (grid[k][0] == 0 and leds[k]) {
				leds[k].maximizeBrightness();
				leds[k] %= dimmer;
				if (j == 20) {
					leds[k] %= 0; //fade all the way to black on last loop
					//Serial.println("Reached last loop and should go to black");
				}
			}

		}
		
		FastLED.show();
		delay(40);
	}

	
	
	//fade in the LEDS that are on in the grid but not already lit up
	for (int l = 0; l < 21; l++) {
		dimmer = dimmer + dimincriment;
		for (int m = 0; m < NUM_LEDS; m++) {
			
			if (grid[m][0] == 1){
				//set the reference pixel to the final value for the location so it can be compared
				refpixel[0] = colors[grid[m][1]]; //apply the color from the list
				refpixel[0].maximizeBrightness();
				refpixel[0] %= (tempdimmer - 1);
				
				// Serial.print("First LED: ");
				// Serial.print(leds[0].getLuma());
				// Serial.print(" Current LED: ");
				// Serial.println(leds[m].getLuma());
				
				if (leds[m].getLuma() < refpixel[0].getLuma()) {
					// Serial.print("LED Number to dim: ");
					// Serial.println(m);
					leds[m] = colors[grid[m][1]];
					leds[m].maximizeBrightness();
					leds[m] %= dimmer;
				}
			}
			
		}
		
		
		FastLED.show();
		delay(40);
	}
	
	//restore the original dimmer setting
	
	dimmer = tempdimmer;
	updatedisplaynormal();
}

void animationfadeall() {
	
	
	
	int tempdimmer = dimmer; //save the previous dimmer setting
	int dimincriment;
	dimincriment = dimmer / 21;
	//fade out
	for (int j = 0; j < 20; j++) {
		dimmer = dimmer - dimincriment;
		updatedisplaynormal();
		delay(40);
	}
	//update time in grid
	cleargrid();
	applycolorstogrid(colorindex);
	refreshtimetogrid();
	
	//fade in
	for (int j = 0; j < 20; j++) {
		dimmer = dimmer + dimincriment;
		updatedisplaynormal();
		delay(40);
	}
	
	//restore the original dimmer setting
	
	dimmer = tempdimmer;
	updatedisplaynormal();
}

void animationzigzag () { //color in all letters, then remove the ones that shouldn't be lit one at a time
      
	  cleargrid();
	  refreshtimetogrid();
	  applycolorstogrid(colorindex);
	  //first light up
      int trail = 7;
      for (int pos = 0; pos < NUM_LEDS + trail; pos++) {

        if (pos < NUM_LEDS + 1) {
          leds[pos] = colors[grid[pos][1]]; //light up the current spot
		  leds[pos].maximizeBrightness();
          leds[pos] %= dimmer;
        }
	
        if (pos > trail) {
          if (grid[pos - trail][0] == 0) { //turn off the trailing x unless it should be on 
            leds[pos - trail] = CRGB::Black;
          }
        }
        
        FastLED.show();
        delay(7);
      }
}

void boop() {
	cleargrid();
	phraseBOOP();
	updatedisplaynormal();
	delay(50);
	cleargrid();
	updatedisplaynormal();
}

void applycolorstogrid(byte colorindex) { //set a new color for all LEDs in the grid
  if (colorindex == 9) {
	 Serial.println("Random");
	 for (int a = 0; a < NUM_LEDS; a++) {
		grid[a][1] = random8(NUM_COLORS);
	 }
  }
  else {
	  //set every position in the array to a single color
	  for (int a = 0; a < NUM_LEDS; a++) {
		grid[a][1] = colorindex;
	  }
  }
}

void cleargrid() { //turn all LEDs to off in the grid
//set every position in the array to off
  for (int b = 0; b < NUM_LEDS; b++) {
    grid[b][0] = 0;
  }
}

void updatedisplaynormal() { //display what is in the grid array, no animation
//copy the grid array onto the leds array, then update the display
  for (int c = 0; c < NUM_LEDS; c++) {
    if(grid[c][0] == 0) {           //if the array says off
      leds[c] = CRGB::Black;        //turn it off/black
    }
    else{
      leds[c] = colors[grid[c][1]]; //apply the color from the list
	  leds[c].maximizeBrightness();
      leds[c] %= dimmer;
    }
  }
  FastLED.show();
}

void gettime() { //update current time variables
//    prevtimemin = mytimemin;
//    Serial.print("previous minute: ");
//    Serial.print(prevtimemin);
//    Serial.print(". ");
    
    DateTime now = rtc.now();
    mytimemonth=now.month();
    mytimeday=now.day();
    mytimehr=now.hour();
    mytimemin=now.minute();
    mytimesec=now.second();
    mydayofweek=now.dayOfTheWeek();

    // Serial.print(now.year(), DEC);
    // Serial.print('/');
    // Serial.print(now.month(), DEC);
    // Serial.print('/');
    // Serial.print(now.day(), DEC);
    // Serial.print(" (");
    // Serial.print(now.hour(), DEC);
    // Serial.print(':');
    // Serial.print(now.minute(), DEC);
    // Serial.print(':');
    // Serial.print(now.second(), DEC);
    // Serial.println(")");
    
}

void refreshtimetogrid() { //puts the current time into the grid array

  //Always on
   phraseITS();

  //calculate minutes on the hour
    if(mytimemin>57 && mytimemin<3){
    }
    // do nothing, no minutes it's on the hour
    
    if(mytimemin>2 && mytimemin<8){
      
      phraseFIVE();
      phrasePAST();
    }
    
    if(mytimemin>7 && mytimemin<13){
      
      phraseTEN();
      phrasePAST();
    }
    if(mytimemin>12 && mytimemin<18){
      
      phraseA();
      phraseQUARTER();
      phrasePAST();
    }
    if(mytimemin>17 && mytimemin<23){
      
      phraseTWENTY();
      phrasePAST();
    }
    if(mytimemin>22 && mytimemin<28){
      
      phraseTWENTY();
      phraseFIVE();
      phrasePAST();
    }
    if(mytimemin>27 && mytimemin<33){
      
      phraseHALF();
      phrasePAST();
    }
    if(mytimemin>32 && mytimemin<38){
      
      phraseTWENTY();
      phraseFIVE();
      phraseTIL();
    }
    if(mytimemin>37 && mytimemin<43){
      
      phraseTWENTY();
      phraseTIL();
    }
    if(mytimemin>42 && mytimemin<48){
      
      phraseA();
      phraseQUARTER();
      phraseTIL();
    }    
    if(mytimemin>47 && mytimemin<53){
      
      phraseTEN();
      phraseTIL();
    }
    if(mytimemin>52 && mytimemin<58){
      
      phraseFIVE();
      phraseTIL();
    }


  // Calculate hour & oclocks
  if(mytimehr==1){
    if(mytimemin>32){
      hourTWO();
      phraseOCLOCK();
    }
    else
    {
      hourONE();
      phraseOCLOCK();
    }
  }
  if(mytimehr==2){
    if(mytimemin>32){
      hourTHREE();
      phraseOCLOCK();
    }
    else
    {
      hourTWO();
      phraseOCLOCK();
    }
  }
    if(mytimehr==3){
    if(mytimemin>32){
      hourFOUR();
      phraseOCLOCK();
    }
    else
    {
      hourTHREE();
      phraseOCLOCK();
    }
  }
  if(mytimehr==4){
    if(mytimemin>32){
      hourFIVE();
      phraseOCLOCK();
    }
    else
    {
      hourFOUR();
      phraseOCLOCK();
    }
  }
  if(mytimehr==5){
    if(mytimemin>32){
      hourSIX();
      phraseOCLOCK();
    }
    else
    {
      hourFIVE();
      phraseOCLOCK();
    }
  }
  if(mytimehr==6){
    if(mytimemin>32){
      hourSEVEN();
      phraseOCLOCK();
    }
    else
    {
      hourSIX();
      phraseOCLOCK();
    }
  }
  if(mytimehr==7){
    if(mytimemin>32){
      hourEIGHT();
      phraseOCLOCK();
    }
    else
    {
      hourSEVEN();
      phraseOCLOCK();
    }
  }
  if(mytimehr==8){
    if(mytimemin>32){
      hourNINE();
      phraseOCLOCK();
    }
    else
    {
      hourEIGHT();
      phraseOCLOCK();
    }
  }
  if(mytimehr==9){
    if(mytimemin>32){
      hourTEN();
      phraseOCLOCK();
    }
    else
    {
      hourNINE();
      phraseOCLOCK();
    }
  }
  if(mytimehr==10){
    if(mytimemin>32){
      hourELEVEN();
      phraseOCLOCK();
    }
    else
    {
      hourTEN();
      phraseOCLOCK();
    }
  }
  if(mytimehr==11){
    if(mytimemin>32){
      hourNOON();
    }
    else
    {
      hourELEVEN();
      phraseOCLOCK();
    }
  }
  if(mytimehr==12){
    if(mytimemin>32){
      hourONE();
      phraseOCLOCK();
    }
    else
    {
      hourNOON();
    }
  }
      if(mytimehr==13){
    if(mytimemin>32){
      hourTWO();
      phraseOCLOCK();
    }
    else
    {
      hourONE();
      phraseOCLOCK();
    }
  }
  if(mytimehr==14){
    if(mytimemin>32){
      hourTHREE();
      phraseOCLOCK();
    }
    else
    {
      hourTWO();
      phraseOCLOCK();
    }
  }
    if(mytimehr==15){
    if(mytimemin>32){
      hourFOUR();
      phraseOCLOCK();
    }
    else
    {
      hourTHREE();
      phraseOCLOCK();
    }
  }
  if(mytimehr==16){
    if(mytimemin>32){
      hourFIVE();
      phraseOCLOCK();
    }
    else
    {
      hourFOUR();
      phraseOCLOCK();
    }
  }
  if(mytimehr==17){
    if(mytimemin>32){
      hourSIX();
      phraseOCLOCK();
    }
    else
    {
      hourFIVE();
      phraseOCLOCK();
    }
  }
  if(mytimehr==18){
    if(mytimemin>32){
      hourSEVEN();
      phraseOCLOCK();
    }
    else
    {
      hourSIX();
      phraseOCLOCK();
    }
  }
  if(mytimehr==19){
    if(mytimemin>32){
      hourEIGHT();
      phraseOCLOCK();
    }
    else
    {
      hourSEVEN();
      phraseOCLOCK();
    }
  }
  if(mytimehr==20){
    if(mytimemin>32){
      hourNINE();
      phraseOCLOCK();
    }
    else
    {
      hourEIGHT();
      phraseOCLOCK();
    }
  }
  if(mytimehr==21){
    if(mytimemin>32){
      hourTEN();
      phraseOCLOCK();
    }
    else
    {
      hourNINE();
      phraseOCLOCK();
    }
  }
  if(mytimehr==22){
    if(mytimemin>32){
      hourELEVEN();
      phraseOCLOCK();
    }
    else
    {
      hourTEN();
      phraseOCLOCK();
    }
  }
  if(mytimehr==23){
    if(mytimemin>32){
      hourMIDNIGHT();
    }
    else
    {
      hourELEVEN();
      phraseOCLOCK();
    }
  }
  if(mytimehr==0){
    if(mytimemin>32){
      hourONE();
      phraseOCLOCK();
    }
    else
    {
      hourMIDNIGHT();
    }
  }
}

void digitstogrid(byte number) { //puts a two digit number into the grid array
	byte firstdigit = number / 10;
	byte seconddigit = number % 10;
	
	// Serial.print(firstdigit);
	// Serial.println(seconddigit);
	
	switch(firstdigit) {
		case 0:
			for (byte i = 1; i<digitzero[0]; i++) {
				grid[digitzero[i]][0] = 1;
			}
		break;
		case 1:
			for (byte i = 1; i<digitone[0]; i++) {
				grid[digitone[i]][0] = 1;
			}
		break;
		case 2:
			for (byte i = 1; i<digittwo[0]; i++) {
				grid[digittwo[i]][0] = 1;
			}
		break;
		case 3:
			for (byte i = 1; i<digitthree[0]; i++) {
				grid[digitthree[i]][0] = 1;
			}
		break;
		case 4:
			for (byte i = 1; i<digitfour[0]; i++) {
				grid[digitfour[i]][0] = 1;
			}
		break;
		case 5:
			for (byte i = 1; i<digitfive[0]; i++) {
				grid[digitfive[i]][0] = 1;
			}
		break;
		case 6:
			for (byte i = 1; i<digitsix[0]; i++) {
				grid[digitsix[i]][0] = 1;
			}
		break;
		case 7:
			for (byte i = 1; i<digitseven[0]; i++) {
				grid[digitseven[i]][0] = 1;
			}
		break;
		case 8:
			for (byte i = 1; i<digiteight[0]; i++) {
				grid[digiteight[i]][0] = 1;
			}
		break;
		case 9:
			for (byte i = 1; i<digitnine[0]; i++) {
				grid[digitnine[i]][0] = 1;
			}
		break;
		
	}
	
	switch(seconddigit) { //shift over 6
		case 0:
			for (byte i = 1; i<digitzero[0]; i++) {
				if (digitzero[i]/11 % 2 == 0) {
					grid[digitzero[i]+6][0] = 1;
				}
				else {
					grid[digitzero[i]-6][0] = 1;
				}
			}
		break;
		case 1:
			for (byte i = 1; i<digitone[0]; i++) {
				if (digitone[i]/11 % 2 == 0) {
					grid[digitone[i]+6][0] = 1;
				}
				else {
					grid[digitone[i]-6][0] = 1;
				}
			}
		break;
		case 2:
			for (byte i = 1; i<digittwo[0]; i++) {
				if (digittwo[i]/11 % 2 == 0) {
					grid[digittwo[i]+6][0] = 1;
				}
				else {
					grid[digittwo[i]-6][0] = 1;
				}
			}
		break;
		case 3:
			for (byte i = 1; i<digitthree[0]; i++) {
				if (digitthree[i]/11 % 2 == 0) {
					grid[digitthree[i]+6][0] = 1;
				}
				else {
					grid[digitthree[i]-6][0] = 1;
				}
			}
		break;
		case 4:
			for (byte i = 1; i<digitfour[0]; i++) {
				if (digitfour[i]/11 % 2 == 0) {
					grid[digitfour[i]+6][0] = 1;
				}
				else {
					grid[digitfour[i]-6][0] = 1;
				}
			}
		break;
		case 5:
			for (byte i = 1; i<digitfive[0]; i++) {
				if (digitfive[i]/11 % 2 == 0) {
					grid[digitfive[i]+6][0] = 1;
				}
				else {
					grid[digitfive[i]-6][0] = 1;
				}
			}
		break;
		case 6:
			for (byte i = 1; i<digitsix[0]; i++) {
				if (digitsix[i]/11 % 2 == 0) {
					grid[digitsix[i]+6][0] = 1;
				}
				else {
					grid[digitsix[i]-6][0] = 1;
				}
			}
		break;
		case 7:
			for (byte i = 1; i<digitseven[0]; i++) {
				if (digitseven[i]/11 % 2 == 0) {
					grid[digitseven[i]+6][0] = 1;
				}
				else {
					grid[digitseven[i]-6][0] = 1;
				}
			}
		break;
		case 8:
			for (byte i = 1; i<digiteight[0]; i++) {
				if (digiteight[i]/11 % 2 == 0) {
					grid[digiteight[i]+6][0] = 1;
				}
				else {
					grid[digiteight[i]-6][0] = 1;
				}
			}
		break;
		case 9:
			for (byte i = 1; i<digitnine[0]; i++) {
				if (digitnine[i]/11 % 2 == 0) {
					grid[digitnine[i]+6][0] = 1;
				}
				else {
					grid[digitnine[i]-6][0] = 1;
				}
			}
		break;
		
	}
	
}

void updatesettimedisplay() {
	switch (setMode) {
		case HH:
			cleargrid();
			grid[6][0] = 1; //light up the h
			digitstogrid(newtimehr);
			updatedisplaynormal();				
		break;
		case MM:
			cleargrid();
			grid[10][0] = 1; //light up the m
			digitstogrid(newtimemin);
			updatedisplaynormal();				
		break;
	}
	
}

//these functions set a group of letters to be on.  

void phraseITS() {
  grid[0][0] = 1;
  grid[1][0] = 1;
  grid[2][0] = 1;
}

void phraseA() {
  grid[4][0] = 1;
}

void phraseHALF() {
  grid[6][0] = 1;
  grid[7][0] = 1;
  grid[8][0] = 1;
  grid[9][0] = 1;
}

void phrasePAST() {
  grid[37][0] = 1;
  grid[36][0] = 1;
  grid[35][0] = 1;
  grid[34][0] = 1;
}

void phraseTIL() {
  grid[40][0] = 1;
  grid[39][0] = 1;
  grid[38][0] = 1;
}

void phraseOCLOCK() {
  grid[115][0] = 1;
  grid[116][0] = 1;
  grid[117][0] = 1;
  grid[118][0] = 1;
  grid[119][0] = 1;
  grid[120][0] = 1;
}

void phraseMONTH() {
  grid[10][0] = 1;
}

void phraseDAY() {
  grid[10][3] = 1;
}

void phraseYEAR() {
  grid[5][0] = 1;
}

void phraseHOUR() {
  grid[6][0] = 1;
}

void phraseMINUTE() {
  grid[10][0] = 1;
  grid[11][0] = 1;
}

void phraseDST() {
  grid[1][0] = 1;
  grid[2][0] = 1;
  grid[3][0] = 1;
}

void phraseFIVE() {
  grid[29][0] = 1;
  grid[30][0] = 1;
  grid[31][0] = 1;
  grid[32][0] = 1;
}

void phraseTEN() {
  grid[20][0] = 1;
  grid[19][0] = 1;
  grid[18][0] = 1;
}

void phraseQUARTER() {
  grid[17][0] = 1;
  grid[16][0] = 1;
  grid[15][0] = 1;
  grid[14][0] = 1;
  grid[13][0] = 1;
  grid[12][0] = 1;
  grid[11][0] = 1;
}

void phraseTWENTY() {
  grid[22][0] = 1;
  grid[23][0] = 1;
  grid[24][0] = 1;
  grid[25][0] = 1;
  grid[26][0] = 1;
  grid[27][0] = 1;
}

void phraseBOOP() {
  grid[64][0] = 1;
  grid[63][0] = 1;
  grid[62][0] = 1;
  grid[61][0] = 1;
}

void hourONE() {
  grid[109][0] = 1;
  grid[108][0] = 1;
  grid[107][0] = 1;
}

void hourTWO() {
  grid[79][0] = 1;
  grid[78][0] = 1;
  grid[77][0] = 1;
}

void hourTHREE() {
  grid[103][0] = 1;
  grid[102][0] = 1;
  grid[101][0] = 1;
  grid[100][0] = 1;
  grid[99][0] = 1;
}

void hourFOUR() {
  grid[110][0] = 1;
  grid[111][0] = 1;
  grid[112][0] = 1;
  grid[113][0] = 1;
}

void hourFIVE() {
  grid[87][0] = 1;
  grid[86][0] = 1;
  grid[85][0] = 1;
  grid[84][0] = 1;
}

void hourSIX() {
  grid[106][0] = 1;
  grid[105][0] = 1;
  grid[104][0] = 1;
}

void hourSEVEN() {
  grid[45][0] = 1;
  grid[46][0] = 1;
  grid[47][0] = 1;
  grid[48][0] = 1;
  grid[49][0] = 1;
}

void hourEIGHT() {
  grid[94][0] = 1;
  grid[95][0] = 1;
  grid[96][0] = 1;
  grid[97][0] = 1;
  grid[98][0] = 1;
}

void hourNINE() {
  grid[83][0] = 1;
  grid[82][0] = 1;
  grid[81][0] = 1;
  grid[80][0] = 1;
}

void hourTEN() {
  grid[74][0] = 1;
  grid[75][0] = 1;
  grid[76][0] = 1;
}

void hourELEVEN() {
  grid[88][0] = 1;
  grid[89][0] = 1;
  grid[90][0] = 1;
  grid[91][0] = 1;
  grid[92][0] = 1;
  grid[93][0] = 1;
}

void hourNOON() {
  grid[51][0] = 1;
  grid[52][0] = 1;
  grid[53][0] = 1;
  grid[54][0] = 1;
}

void hourMIDNIGHT() {
  grid[66][0] = 1;
  grid[67][0] = 1;
  grid[68][0] = 1;
  grid[69][0] = 1;
  grid[70][0] = 1;
  grid[71][0] = 1;
  grid[72][0] = 1;
  grid[73][0] = 1;
}
